import os
import unittest, doctest
from zope.app.testing import functional
from zope.testing import module, cleanup

ResourceLibraryFunctionalLayer = functional.ZCMLLayer(
    os.path.join(os.path.split(__file__)[0], 'ftesting.zcml'),
    __name__, 'ResourceLibraryFunctionalLayer')
    
def setUp(test):
    module.setUp(test, 'megrok.resourcelibrary.tests')

def tearDown(test):
    module.tearDown(test)
    cleanup.cleanUp()

def test_suite():
    suite = functional.FunctionalDocFileSuite(
        '../README.txt',
        optionflags=doctest.NORMALIZE_WHITESPACE + doctest.ELLIPSIS,
        setUp=setUp,
        tearDown=tearDown,
        )
    suite.layer = ResourceLibraryFunctionalLayer
    return unittest.TestSuite([
        suite,
        ])



import grok
import zc.resourcelibrary
from megrok.resourcelibrary.directive import default_library_name

class ResourceLibrary(object):
    @classmethod
    def need(cls):
        name = grok.name.bind(get_default=default_library_name).get(cls)
        zc.resourcelibrary.need(name)



import os

import martian
import grok
from martian.error import GrokError

from zope.publisher.interfaces.browser import (IDefaultBrowserLayer,
                                               IBrowserRequest)
from zope.app.publisher.browser import directoryresource
from zope.app.publisher.browser.resourcemeta import allowed_names
from zope.security.checker import CheckerPublic, NamesChecker
from zc.resourcelibrary.zcml import handler
from zc.resourcelibrary.resourcelibrary import LibraryInfo, library_info
from zope.interface import Interface

from megrok import resourcelibrary
from megrok.resourcelibrary.directive import default_list, default_library_name

class ResourceLibraryGrokker(martian.ClassGrokker):
    martian.component(resourcelibrary.ResourceLibrary)
    martian.directive(grok.name, get_default=default_library_name)
    martian.directive(resourcelibrary.directory)
    martian.directive(resourcelibrary.use, get_default=default_list)
    martian.directive(resourcelibrary.include, get_default=default_list)
    martian.directive(grok.layer, default=IDefaultBrowserLayer)
    martian.directive(grok.require, name='permission')
    
    def grok(self, name, factory, module_info, **kw):
        # need to tuck away module_info
        factory.module_info = module_info
        return super(ResourceLibraryGrokker, self).grok(
            name, factory, module_info, **kw)
    
    def execute(self, class_, config, name, directory, use, include, layer,
                permission,
                **kw):
        directory = class_.module_info.getResourcePath(directory)
        if not os.path.isdir(directory):
            raise GrokImportError(
                "You can only pass a directory to the '%s' directive." %
                self.name)

        library_info[name] = LibraryInfo()
        library_info[name].required.extend(use)
        library_info[name].included.extend(include)

        if permission == 'zope.Public' or permission is None:
            permission = CheckerPublic
        checker = NamesChecker(allowed_names, permission)

        factory = directoryresource.DirectoryResourceFactory
        factory = factory(directory, checker, name)
        
        config.action(
            discriminator=('resource', name, IBrowserRequest, layer),
            callable=handler,
            args=(name, library_info[name].required, (layer,),
                  Interface, name, factory, config.info),
            )
        return True



from setuptools import setup, find_packages
import os

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = (
    read('src/megrok/resourcelibrary/README.txt')
    + '\n' +
    read('CHANGES.txt')
    )

setup(name='megrok.resourcelibrary',
      version='0.9.1',
      description="Static resource library support for Grok.",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Grok Team',
      author_email='grok-dev@zope.org',
      url='',
      license='ZPL',
      packages=find_packages('src'),
      package_dir = {'': 'src'},
      namespace_packages=['megrok'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'grok >= 0.13',
          'zc.resourcelibrary',
         ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



import os

import martian
from martian import util
from martian.error import GrokImportError

from zc.resourcelibrary.zcml import INCLUDABLE_EXTENTIONS

def default_list(factory, module=None, **data):
    return []

def default_library_name(factory, module=None, **data):
    return factory.__name__.lower()

class directory(martian.Directive):
    scope = martian.CLASS
    store = martian.ONCE

    validate = martian.validateText
    
class use(martian.Directive):
    scope = martian.CLASS
    store = martian.MULTIPLE
    validate = martian.validateText
    
class include(martian.Directive):
    scope = martian.CLASS
    store = martian.MULTIPLE
    
    def validate(self, value):
        if util.not_unicode_or_ascii(value):
            raise GrokImportError(
                "You can only pass unicode or ASCII to the '%s' directive." %
                self.name)
        ext = os.path.splitext(value)[1]
        if ext not in INCLUDABLE_EXTENTIONS:
            raise GrokImportError(
                "The '%s' directive does not know how to include this file: %s" %
                (value, self.name))

